g=love.graphics


local oscs={}
local oscv={1,1,1,1,1}

local dif=-5
local pitch=1
local pitches={1,1.059,1.122,1.189,1.260,1.335,1.414,1.498,1.587,
   1.682,1.782,1.888}

local envstate="none"
local vol=0

local env,filter

function love.load()
 require("knob");
 
 for i=1, #oscv do
  local o=love.audio.newSource("saw.ogg",true)
  o:setLooping(true)
  o:setPitch(2^(i+dif))
  table.insert(oscs,o)
  love.audio.play(o);
 end
 
 env={attack=Knob.new(5,35,0.01,1,"Attack",1),
      decay= Knob.new(5+80,35,0,1,"Decay",0),
      release=Knob.new(5+160,35,0.05,1,"Release",1)}
 filter={slope=Knob.new(5,500,-1,1,"Decrease"),
	 stall=Knob.new(5+80,500,0,#oscv,"Stall",0),
	 base=Knob.new(5+160,500,0,1,"Base",1)}
 
 love.update()
end

function love.update()
 for i,v in pairs(knobs) do v.update() end
 if envstate=="attack" then
  vol=vol+env.attack.value
  if vol>=1 then vol=1 envstate="decay" end
 elseif envstate=="decay" or envstate=="release" then
  vol=vol-env[envstate].value
  if vol<=0 then vol=0 envstate="none" end
 end
 for i,v in pairs(oscs) do
  v:setVolume((oscv[i]/#oscs)*vol)
  v:setPitch(pitch*(2^(i+dif)))
 end
 
 local spo=10
 
 local h=filter.base.value
 for i=1, #oscv*spo do
  if i>filter.stall.value*spo then
   h=h-(filter.slope.value/spo)
  end
  if (i-1)%spo==0 then
   local at=((i-1)/spo)+1
   oscv[at]=math.max(math.min(h,1),0)
  end
 end
end

local oscboxw=400
local oscboxh=300

function love.draw()
 g.setBackgroundColor(30,30,30);
 love.graphics.setFont(20);
 love.graphics.setColor(255,127,0);
 love.graphics.print("Saw synth",5,5);
 love.graphics.setColor(255,255,255);
 love.graphics.line(0,30,g.getWidth(),30);
 for i,v in pairs(knobs) do v.draw() end
 
 local x=(g.getWidth()-oscboxw)/2
 local y=150
 g.setColor(0,0,0)
 g.rectangle("fill",x,y,oscboxw,oscboxh)
 g.setColor(255,128,0)
 g.rectangle("line",x,y,oscboxw,oscboxh)
 
 g.setFont(30);
 g.print("Volume Envelope",300,60);
 g.printf("Filter",x,y+oscboxh,oscboxw,"center");
 
 local each=oscboxw/#oscv
 for i,v in pairs(oscv) do
  local xp=x+((each/2)+each*(i-1))
  love.graphics.setColor(255,256-math.floor(256*v),0)
  love.graphics.line(xp,y+oscboxh,xp,(y+oscboxh)-(oscboxh*v*0.7))
 end
end

local keys={"z","s","x","d","c","v","g","b","h","n","j","m",","}
local keypressing=nil
function love.keypressed(k,u)
 for i,v in pairs(keys) do
  if v==k then
   local oct=1
   while i>12 do oct=oct+1 i=(i-12) end
   pitch=pitches[1+((i-1)%12)]*oct
   keypressing=k
   envstate="attack"
   vol=0
  end
 end
end
function love.keyreleased(k,u)
 if keypressing==k then
  envstate="release"
 end
end